@@ -29,19 +29,20 @@
 
 #include <tumblerd/tumbler-registry.h>
 #include <tumblerd/tumbler-specialized-thumbnailer.h>
+#include <tumblerd/tumbler-utils.h>
 
 
 
-static void         tumbler_registry_finalize                  (GObject            *object);
-static void         tumbler_registry_remove_thumbnailer        (const gchar        *key,
-                                                                GList             **list,
-                                                                TumblerThumbnailer *thumbnailer);
-static void         tumbler_registry_list_free                 (gpointer            data);
-static GList       *tumbler_registry_get_thumbnailers_internal (TumblerRegistry    *registry);
-static gint         tumbler_registry_compare                   (TumblerThumbnailer *a,
-                                                                TumblerThumbnailer *b);
-TumblerThumbnailer *tumbler_registry_lookup                    (TumblerRegistry    *registry,
-                                                                const gchar        *hash_key);
+static void                tumbler_registry_finalize                  (GObject            *object);
+static void                tumbler_registry_remove_thumbnailer        (const gchar        *key,
+                                                                       GList             **list,
+                                                                       TumblerThumbnailer *thumbnailer);
+static void                tumbler_registry_list_free                 (gpointer            data);
+static GList              *tumbler_registry_get_thumbnailers_internal (TumblerRegistry    *registry);
+static gint                tumbler_registry_compare                   (TumblerThumbnailer *a,
+                                                                       TumblerThumbnailer *b);
+static TumblerThumbnailer *tumbler_registry_lookup                    (TumblerRegistry    *registry,
+                                                                       const gchar        *hash_key);
 
 
 
@@ -56,7 +57,7 @@ struct _TumblerRegistry
 
   GHashTable   *thumbnailers;
   GHashTable   *preferred_thumbnailers;
-  GMutex       *mutex;
+  TUMBLER_MUTEX (mutex);
 
   gchar       **uri_schemes;
   gchar       **mime_types;
@@ -90,7 +91,7 @@ tumbler_registry_class_init (TumblerRegistryClass *klass)
 static void
 tumbler_registry_init (TumblerRegistry *registry)
 {
-  registry->mutex = g_mutex_new ();
+  tumbler_mutex_create (registry->mutex);
   registry->thumbnailers = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                   g_free, tumbler_registry_list_free);
   registry->preferred_thumbnailers = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -113,7 +114,7 @@ tumbler_registry_finalize (GObject *object)
   g_strfreev (registry->mime_types);
 
   /* destroy the mutex */
-  g_mutex_free (registry->mutex);
+  tumbler_mutex_free (registry->mutex);
 
   (*G_OBJECT_CLASS (tumbler_registry_parent_class)->finalize) (object);
 }
@@ -124,36 +125,10 @@ tumbler_registry_finalize (GObject *object)
 static void
 dump_registry (TumblerRegistry *registry)
 {
-  TumblerThumbnailer *thumbnailer;
-  GHashTableIter      iter;
-  const gchar        *hash_key;
-  GList             **thumbnailers;
-  GList              *lp;
-
   g_print ("Registry:\n");
-
-  g_print ("  Preferred Thumbnailers:\n");
-  g_hash_table_iter_init (&iter, registry->preferred_thumbnailers);
-  while (g_hash_table_iter_next (&iter, (gpointer) &hash_key, (gpointer) &thumbnailer))
-    {
-      g_print ("    %s: %s\n", hash_key,
-               tumbler_specialized_thumbnailer_get_name (TUMBLER_SPECIALIZED_THUMBNAILER (thumbnailer)));
-    }
-
-  g_print ("  Registry Thumbnailers:\n");
-  g_hash_table_iter_init (&iter, registry->thumbnailers);
-  while (g_hash_table_iter_next (&iter, (gpointer) &hash_key, (gpointer) &thumbnailers))
-    {
-      for (lp = *thumbnailers; lp != NULL; lp = lp->next)
-        {
-          if (TUMBLER_IS_SPECIALIZED_THUMBNAILER (lp->data))
-            {
-              g_print ("    %s: %s\n", 
-                       hash_key, tumbler_specialized_thumbnailer_get_name (lp->data));
-            }
-        }
-    }
-
+  g_print ("  Preferred Thumbnailers: %d\n",
+           g_hash_table_size (registry->preferred_thumbnailers));
+  g_print ("  Registry Thumbnailers: %d\n",g_hash_table_size (registry->thumbnailers));
   g_print ("\n");
 }
 #endif
@@ -285,7 +260,7 @@ tumbler_registry_get_thumbnailers_internal (TumblerRegistry *registry)
 
 
 
-TumblerThumbnailer *
+static TumblerThumbnailer *
 tumbler_registry_lookup (TumblerRegistry *registry,
                          const gchar     *hash_key)
 {
@@ -348,7 +323,7 @@ tumbler_registry_add (TumblerRegistry    *registry,
   g_return_if_fail (TUMBLER_IS_REGISTRY (registry));
   g_return_if_fail (TUMBLER_IS_THUMBNAILER (thumbnailer));
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
 
   /* determine the hash keys (all combinations of URI schemes and MIME types)
    * for this thumbnailer */
@@ -390,7 +365,7 @@ tumbler_registry_add (TumblerRegistry    *registry,
   dump_registry (registry);
 #endif
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 }
 
 
@@ -402,7 +377,7 @@ tumbler_registry_remove (TumblerRegistry    *registry,
   g_return_if_fail (TUMBLER_IS_REGISTRY (registry));
   g_return_if_fail (TUMBLER_IS_THUMBNAILER (thumbnailer));
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
 
   g_signal_handlers_disconnect_matched (thumbnailer, G_SIGNAL_MATCH_DATA, 
                                         0, 0, NULL, NULL, registry);
@@ -411,7 +386,7 @@ tumbler_registry_remove (TumblerRegistry    *registry,
   g_hash_table_foreach (registry->thumbnailers, 
                         (GHFunc) tumbler_registry_remove_thumbnailer, thumbnailer);
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 }
 
 
@@ -423,11 +398,11 @@ tumbler_registry_get_thumbnailers (TumblerRegistry *registry)
 
   g_return_val_if_fail (TUMBLER_IS_REGISTRY (registry), NULL);
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
 
   thumbnailers = tumbler_registry_get_thumbnailers_internal (registry);
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 
   /* return all active thumbnailers */
   return thumbnailers;
@@ -456,7 +431,7 @@ tumbler_registry_get_thumbnailer_array (TumblerRegistry    *registry,
   /* iterate over all URIs */
   for (n = 0; n < length; ++n)
     {
-      g_mutex_lock (registry->mutex);
+      tumbler_mutex_lock (registry->mutex);
 
       /* determine the URI scheme and generate a hash key */
       scheme = g_uri_parse_scheme (tumbler_file_info_get_uri (infos[n]));
@@ -470,7 +445,7 @@ tumbler_registry_get_thumbnailer_array (TumblerRegistry    *registry,
       g_free (hash_key);
       g_free (scheme);
 
-      g_mutex_unlock (registry->mutex);
+      tumbler_mutex_unlock (registry->mutex);
     }
 
   /* NULL-terminate the array */
@@ -494,11 +469,11 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
 {
   GHashTableIter iter;
   GHashTable    *unique_pairs;
-  GSList        *used_strings = NULL;
+  GPtrArray     *used_strings;
   GList         *thumbnailers;
   GList         *lp;
   const gchar  **pair;
-  GString       *pair_string;
+  gchar         *pair_string;
   gchar        **mime_types;
   gchar        **uri_schemes;
   gint           n;
@@ -506,7 +481,7 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
 
   g_return_if_fail (TUMBLER_IS_REGISTRY (registry));
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
 
   /* free the old cache */
   g_strfreev (registry->uri_schemes);
@@ -517,7 +492,7 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
   /* get a list of all active thumbnailers */
   thumbnailers = tumbler_registry_get_thumbnailers_internal (registry);
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 
   /* abort if there are no thumbnailers */
   if (thumbnailers == NULL)
@@ -532,6 +507,9 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
                                         (GDestroyNotify) g_free, 
                                         (GDestroyNotify) free_pair);
 
+  /* prepare array */
+  used_strings = g_ptr_array_sized_new (150);
+
   /* iterate over all of them */
   for (lp = thumbnailers; lp != NULL; lp = lp->next)
     {
@@ -548,7 +526,7 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
            ++n)
         {
           /* remember the MIME type so that we can later reuse it without copying */
-          used_strings = g_slist_prepend (used_strings, mime_types[n]);
+          g_ptr_array_add (used_strings, mime_types[n]);
 
           for (u = 0; uri_schemes[u] != NULL; ++u)
             {
@@ -556,7 +534,7 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
                * without copying. Only remember it once (n==0) to avoid segmentation 
                * faults when freeing the list */
               if (n == 0)
-                used_strings = g_slist_prepend (used_strings, uri_schemes[u]);
+                g_ptr_array_add (used_strings, uri_schemes[u]);
 
               /* allocate a pair with the current URI scheme and MIME type */
               pair = g_slice_alloc (2 * sizeof (const gchar *));
@@ -566,15 +544,10 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
               pair[1] = mime_types[n];
 
               /* combine the two to a unique pair identifier */
-              pair_string = g_string_new (pair[0]);
-              g_string_append_c (pair_string, '-');
-              g_string_append (pair_string, pair[1]);
+              pair_string = g_strdup_printf ("%s-%s", pair[0], pair[1]);
 
               /* remember the pair in the hash table */
-              g_hash_table_insert (unique_pairs, pair_string->str, pair);
-
-              /* free the pair string */
-              g_string_free (pair_string, FALSE);
+              g_hash_table_insert (unique_pairs, pair_string, pair);
             }
         }
 
@@ -595,7 +568,7 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
 
   n = g_hash_table_size (unique_pairs);
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
 
   /* allocate a string array for the URI scheme / MIME type pairs */
   registry->uri_schemes = g_new0 (gchar *, n+1);
@@ -617,14 +590,14 @@ tumbler_registry_update_supported (TumblerRegistry *registry)
   registry->uri_schemes[n] = NULL;
   registry->mime_types[n] = NULL;
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 
   /* destroy the hash table we used */
   g_hash_table_unref (unique_pairs);
 
   /* free all strings we used but haven't freed yet */
-  g_slist_foreach (used_strings, (GFunc) g_free, NULL);
-  g_slist_free (used_strings);
+  g_ptr_array_foreach (used_strings, (GFunc) g_free, NULL);
+  g_ptr_array_free (used_strings, TRUE);
 }
 
 
@@ -636,7 +609,7 @@ tumbler_registry_get_supported (TumblerRegistry     *registry,
 {
   g_return_if_fail (TUMBLER_IS_REGISTRY (registry));
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
   
   if (uri_schemes != NULL)
     *uri_schemes = (const gchar *const *)registry->uri_schemes;
@@ -644,7 +617,7 @@ tumbler_registry_get_supported (TumblerRegistry     *registry,
   if (mime_types != NULL)
     *mime_types = (const gchar *const *)registry->mime_types;
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 }
 
 
@@ -658,9 +631,9 @@ tumbler_registry_get_preferred (TumblerRegistry *registry,
   g_return_val_if_fail (TUMBLER_IS_REGISTRY (registry), NULL);
   g_return_val_if_fail (hash_key != NULL && *hash_key != '\0', NULL);
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
   thumbnailer = g_hash_table_lookup (registry->preferred_thumbnailers, hash_key);
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 
   return thumbnailer != NULL ? g_object_ref (thumbnailer) : NULL;
 }
@@ -676,7 +649,7 @@ tumbler_registry_set_preferred (TumblerRegistry    *registry,
   g_return_if_fail (hash_key != NULL && *hash_key != '\0');
   g_return_if_fail (thumbnailer == NULL || TUMBLER_IS_THUMBNAILER (thumbnailer));
 
-  g_mutex_lock (registry->mutex);
+  tumbler_mutex_lock (registry->mutex);
   
   if (thumbnailer == NULL)
     {
@@ -692,5 +665,5 @@ tumbler_registry_set_preferred (TumblerRegistry    *registry,
   dump_registry (registry);
 #endif
 
-  g_mutex_unlock (registry->mutex);
+  tumbler_mutex_unlock (registry->mutex);
 }
